// -*- C++ -*-
/*!
 * @file  PA10SystemController.cpp
 * @brief PA10システム制御
 * @date $Date$
 *
 * $Id$
 */

#include "PA10SystemController.h"

// Module specification
// <rtc-template block="module_spec">
static const char* pa10systemcontroller_spec[] =
  {
    "implementation_id", "PA10SystemController",
    "type_name",         "PA10SystemController",
    "description",       "PA10システム制御",
    "version",           "1.0.0",
    "vendor",            "FSI",
    "category",          "PA10SystemController",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "0",
    "language",          "C++",
    "lang_type",         "compile",
    "conf.default.scenario", "cd ~/PCforSystemControl/bin/script; python PA10DrinkServe.py %d",
    "conf.default.check", "cd ~/PCforSystemControl/bin/script; python RTCAliveCheck.py",
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
PA10SystemController::PA10SystemController(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_recogRH_dataIn("RecogRHData", m_recogRH_data),
    m_recogDrink_dataIn("RecogDrinkData", m_recogDrink_data),
    m_PA10SysCtrlrProvPort("PA10SysCtrlrProv"),
    m_ControlTerminalConsPort("ControlTerminalCons"),
    m_ComRMRCConsPort("ComRMRCCons"),
    m_ComHandConsPort("ComHandCons"),
    m_RecogRHConsPort("RecogRHCons"),
    m_RecogDrinkConsPort("RecogDrinkCons")

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
PA10SystemController::~PA10SystemController()
{
}



RTC::ReturnCode_t PA10SystemController::onInitialize()
{

  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("RecogRHData", m_recogRH_dataIn);
  addInPort("RecogDrinkData", m_recogDrink_dataIn);
  
  // Set OutPort buffer
 
  // Set service provider to Ports
  m_PA10SysCtrlrProvPort.registerProvider("SystemControl", "PA10SystemControl", m_system_control);
 
  // Set service consumers to Ports
  m_ControlTerminalConsPort.registerConsumer("StateChangeFromPA10", "StateChangeFromPA10", m_state_change_from_pa10);
  m_ComRMRCConsPort.registerConsumer("com_frm_ctrl", "ComFrmCtrl", m_com_frm_ctrl);
  m_ComRMRCConsPort.registerConsumer("com_fk", "ComFk", m_com_fk);
  m_ComRMRCConsPort.registerConsumer("com_move", "ComMove", m_com_move);
  m_ComHandConsPort.registerConsumer("com_hand", "ComHand", m_com_hand);
  m_RecogRHConsPort.registerConsumer("recogRH", "RecognitionService", m_recogRH);
  m_RecogDrinkConsPort.registerConsumer("recogDrink", "RecognitionService", m_recogDrink);

  // Set CORBA Service Ports
  addPort(m_PA10SysCtrlrProvPort);
  addPort(m_ControlTerminalConsPort);
  addPort(m_ComRMRCConsPort);
  addPort(m_ComHandConsPort);
  addPort(m_RecogRHConsPort);
  addPort(m_RecogDrinkConsPort);
  
  // </rtc-template>

  bindParameter("scenario", m_scenario, "cd ~/PCforSystemControl/bin/script; python PA10DrinkServe.py %d");
  bindParameter("check", m_check, "cd ~/PCforSystemControl/bin/script; python RTCAliveCheck.py");
  //スレッドID
  //m_system_control.threadid = 1;	//給仕作業用

  //状態リスト
  strcpy(&m_system_control.PA10state_list[0][0], "待機中");
  strcpy(&m_system_control.PA10state_list[1][0], "起動中");
  strcpy(&m_system_control.PA10state_list[2][0], "給仕中");
  strcpy(&m_system_control.PA10state_list[3][0], "終了");
  strcpy(&m_system_control.PA10state_list[4][0], "給仕完了");
  
  //エラーリスト
  strcpy(&m_system_control.PA10err_list[0][0], "異常なし。");
  strcpy(&m_system_control.PA10err_list[1][0], "システム内RTCが全て起動していません。");
  strcpy(&m_system_control.PA10err_list[2][0], "給仕シナリオ実行スレッド終了に失敗しました。");
  strcpy(&m_system_control.PA10err_list[3][0], "PA10の状態が未定義の状態です。");
  strcpy(&m_system_control.PA10err_list[4][0], "実機エラーが発生しました。");

  red_flag = 0;
  
  return RTC::RTC_OK;
}


RTC::ReturnCode_t PA10SystemController::onFinalize()
{
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t PA10SystemController::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t PA10SystemController::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t PA10SystemController::onActivated(RTC::UniqueId ec_id)
{

  coil::TimeValue tv;

  //使用するスクリプト
  strcpy(m_system_control.scenario_script, m_scenario.c_str());
  strcpy(m_system_control.check_script, m_check.c_str());
  
  //PA10システムの起動
  //状態とエラーの初期化
  m_system_control.PA10state = 1;
  m_system_control.PA10err = 0;
    
  return RTC::RTC_OK;
}


RTC::ReturnCode_t PA10SystemController::onDeactivated(RTC::UniqueId ec_id)
{

  //状態遷移によるDeactivateでない場合(例えば、RTC_ERROR発生時)のための終了処理を書いておく
  //状態変更（→　終了）
  if (m_system_control.PA10state != 3){
    m_system_control.PA10SetStatus(3);
    if (::CORBA::is_nil(m_state_change_from_pa10.getObject()) == false){
      m_state_change_from_pa10->SetState(m_system_control.PA10state, &m_system_control.PA10state_list[m_system_control.PA10state][0]);
    }else {
    	RTC_INFO(("port connect error"));
    }
    //RTCの終了
    m_system_control.PA10End();
  }
  red_flag = 0;
  
  return RTC::RTC_OK;
}


RTC::ReturnCode_t PA10SystemController::onExecute(RTC::UniqueId ec_id)
{

  coil::TimeValue tv;
  void* ret = NULL;
  switch(m_system_control.PA10state){
        case 1://制御端末へ状態変更通知（→　起動中）
              if (::CORBA::is_nil(m_state_change_from_pa10.getObject()) == false){
                m_state_change_from_pa10->SetState(m_system_control.PA10state, &m_system_control.PA10state_list[m_system_control.PA10state][0]);
              }else {
              	RTC_INFO(("port connect error"));
                }

                //起動済みRTCのチェック
              RTCActiveCheck(m_check.c_str());
              coil::usleep(1000000);
              
                //時刻計測
              tv = coil::gettimeofday();
              old_tm.sec = tv.sec();

        case 0:{//PA10システムの監視(10秒毎)
              
              tv = coil::gettimeofday();
              start_tm.sec = tv.sec();
              if(calc_tm_dif(start_tm, old_tm) > 10){
              
                //制御端末へ状態変更通知（→　待機中）
                if(red_flag == 0){//遷移直後に一度だけ通知
                  if (::CORBA::is_nil(m_state_change_from_pa10.getObject()) == false){
                    m_state_change_from_pa10->SetState(m_system_control.PA10state, &m_system_control.PA10state_list[m_system_control.PA10state][0]);
                    red_flag = 1;
                  }else {
                    RTC_INFO(("port connect error"));
                  }
                }
                
                //起動済みRTCのチェック
                RTCActiveCheck(m_check.c_str());
                  
                /* 
                 * エラーチェック
                 * エラーが生じていた場合はシステムを終了させる
                 */
                if(m_system_control.PA10err != 0){
                  //状態変更（待機　→　終了）
                m_system_control.PA10SetStatus(3);
                }
                old_tm.sec = start_tm.sec;
              }
              break;
          }  
        case 2://給仕中
              if(sv_flag == 0){
                  //制御端末へ状態変更通知（→　給仕中）
                if (::CORBA::is_nil(m_state_change_from_pa10.getObject()) == false){
                  m_state_change_from_pa10->SetState(m_system_control.PA10state, &m_system_control.PA10state_list[m_system_control.PA10state][0]);
                  sv_flag = 1;
                }else {
                  RTC_INFO(("port connect error"));
                  }
                }
              break;
              
        case 3://PA10システムの終了
                //制御端末へ状態変更通知（→　終了）
              if (::CORBA::is_nil(m_state_change_from_pa10.getObject()) == false){
                m_state_change_from_pa10->SetState(m_system_control.PA10state, &m_system_control.PA10state_list[m_system_control.PA10state][0]);
              }else {
                RTC_INFO(("port connect error"));
              }
                
              //RTCの終了
              m_system_control.PA10End();
                
              //PA10システム非活性化
              //PA10SystemController::onDeactivated(ec_id);
              
              red_flag = 0;
              break;
              
        case 4://給仕完了
                //制御端末へ状態変更通知（→　給仕完了）
              if (::CORBA::is_nil(m_state_change_from_pa10.getObject()) == false){
                m_state_change_from_pa10->SetState(m_system_control.PA10state, &m_system_control.PA10state_list[m_system_control.PA10state][0]);
              }else {
                RTC_INFO(("port connect error"));
              }
                //ワーカスレッドを終了させる
              if(pthread_join(m_system_control.threadid, &ret)){
                std::cout << "pthread_joint error!" << std::endl;
                RTC_INFO(("pthread_joint error!"));
                
                  //ワーカスレッドを終了失敗
                m_system_control.PA10ReportError(2);
                
                  //状態変更（→　終了）
                m_system_control.PA10SetStatus(3);
                return RTC::RTC_ERROR;
                }
                //状態変更（→　待機中）
              m_system_control.PA10SetStatus(0);
              sv_flag = 0;
              red_flag = 0;
              break;
         
        default:
                //未定義の状態
              m_system_control.PA10ReportError(3);
              
                //状態変更（→　終了）
              m_system_control.PA10SetStatus(3);
              break;
  }
  return RTC::RTC_OK;
}


RTC::ReturnCode_t PA10SystemController::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t PA10SystemController::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t PA10SystemController::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t PA10SystemController::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t PA10SystemController::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/* 
 * 起動済みRTCをチェックするスクリプトを実行
 * スクリプト内処理でPA10の状態を１または３へと変更する
 */
void RTCActiveCheck(const char* check_script){

  char cmd[128];
  sprintf( cmd, check_script);
  system(cmd);

}

//-------------------------------------------------------------
// helper functions
//-------------------------------------------------------------
float calc_tm_dif(RTC::Time tm1, RTC::Time tm2) {
  return (float)(tm1.sec - tm2.sec)
    + (float)(1000000000 + tm1.nsec - tm2.nsec)/ 1000000000.0 - 1.0;
}

extern "C"
{
 
  void PA10SystemControllerInit(RTC::Manager* manager)
  {
    coil::Properties profile(pa10systemcontroller_spec);
    manager->registerFactory(profile,
                             RTC::Create<PA10SystemController>,
                             RTC::Delete<PA10SystemController>);
  }
  
};


