// -*-C++-*-
/*!
 * @file  PA10SystemControllerSVC_impl.h
 * @brief Service implementation header of PA10SystemController.idl
 *
 */

#include "PA10SystemControllerSkel.h"
#include <rtm/SystemLogger.h>

#ifndef PA10SYSTEMCONTROLLERSVC_IMPL_H
#define PA10SYSTEMCONTROLLERSVC_IMPL_H

#define STR_MAX 50

#include <stdio.h>
#include <stdlib.h>
#include <pthread.h> 
#include <iostream>

void* thread_main(void* num);

/*!
 * @class PA10SystemControlSVC_impl
 * Example class implementing IDL interface PA10SystemControl
 */
class PA10SystemControlSVC_impl
 : public virtual POA_PA10SystemControl,
   public virtual PortableServer::RefCountServantBase
{
 private:
   // Make sure all instances are built on the heap by making the
   // destructor non-public
   //virtual ~PA10SystemControlSVC_impl();

 public:
  /*!
   * @brief standard constructor
   */
   PA10SystemControlSVC_impl();
  /*!
   * @brief destructor
   */
   virtual ~PA10SystemControlSVC_impl();

   // attributes and operations
   CORBA::Long PA10GetStatus();
   void PA10SetStatus(CORBA::Long state);
   void PA10End();
   void PA10DrinkServeReq(CORBA::Short drinknum);
   void PA10ReportError(CORBA::Long errnum);
   RTC::Logger rtclog;
   
   /*
   * state of PA10
    * 0:起動中
    * 1:待機中
    * 2:給仕中
    * 3:終了
    * 4:給仕完了
    */
   CORBA::Long PA10state;
   
   /*
   * error of PA10
    *    0:異常なし。
    *    1:システム内RTCが全て起動していません。
    *    2:給仕シナリオ実行スレッド終了に失敗しました。
    *   3:PA10の状態が未定義の状態です。
    *    4:実機エラーが発生しました。
    */
   CORBA::Long PA10err;
   
   /*
    * thread ID 
    *   現在はスレッドとして立ち上げる処理は給仕作業のみ
    *   一旦給仕作業が終了するまで、次の給仕要求は受け付けない
    */
   pthread_t threadid;
   //short d_num;
   char args[128];
   char scenario_script[127];
   char check_script[128];
   char PA10state_list[5][STR_MAX];
   char PA10err_list[5][STR_MAX];

};



#endif // PA10SYSTEMCONTROLLERSVC_IMPL_H


