// -*- C++ -*-
/*!
 * @file  ACT_PA10_RH.cpp * @brief ACT_RTC 中レベルI/FによるPA10,RHアーム制御モジュール * $Date$ 
 *
 * $Id$ 
 */
#include "ACT_PA10_RH.h"

// Module specification
// <rtc-template block="module_spec">
static const char* act_pa10_rh_spec[] =
  {
    "implementation_id", "ACT_PA10_RH",
    "type_name",         "ACT_PA10_RH",
    "description",       "ACT_RTC 中レベルI/FによるPA10,RHアーム制御モジュール",
    "version",           "1.0.0",
    "vendor",            "RTC-Center",
    "category",          "ACT_RTC",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "0",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    ""
  };
// </rtc-template>

ACT_PA10_RH::ACT_PA10_RH(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_MiddleProvPort("MiddleProv"),
    m_CommonProvPort("CommonProv"),
    m_ACTProvPort("ACTProv"),
    m_RMRCConsPort("RMRCCons"),
    m_HandConsPort("HandCons")

    // </rtc-template>
{
}

ACT_PA10_RH::~ACT_PA10_RH()
{
}


RTC::ReturnCode_t ACT_PA10_RH::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers

  // Set OutPort buffer

  // Set service provider to Ports
  m_MiddleProvPort.registerProvider("ManipulatorCommonInterface_Middle", "ManipulatorCommonInterface_Middle", m_ManipulatorCommonInterface_Middle);
  m_CommonProvPort.registerProvider("ManipulatorCommonInterface_Common", "ManipulatorCommonInterface_Common", m_ManipulatorCommonInterface_Common);
  m_ACTProvPort.registerProvider("com_ACT", "ComACT", m_com_ACT);

  // Set service consumers to Ports
  m_RMRCConsPort.registerConsumer("com_frm_ctrl", "ComFrmCtrl", m_com_frm_ctrl);
  m_RMRCConsPort.registerConsumer("com_fk", "ComFk", m_com_fk);
  m_RMRCConsPort.registerConsumer("com_move", "ComMove", m_com_move);
  m_HandConsPort.registerConsumer("com_hand", "ComHand", m_com_hand);

  // Set CORBA Service Ports
  addPort(m_MiddleProvPort);
  addPort(m_CommonProvPort);
  addPort(m_ACTProvPort);
  addPort(m_RMRCConsPort);
  addPort(m_HandConsPort);

  // resist consumer address
  m_ManipulatorCommonInterface_Middle.adrs_com_frm_ctrl_cons = &m_com_frm_ctrl;
  m_ManipulatorCommonInterface_Middle.adrs_com_fk_cons       = &m_com_fk;
  m_ManipulatorCommonInterface_Middle.adrs_com_move_cons     = &m_com_move;
  m_ManipulatorCommonInterface_Middle.adrs_com_hand_cons     = &m_com_hand;
  m_ManipulatorCommonInterface_Middle.j_num = 0;
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable

  // </rtc-template>
  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t ACT_PA10_RH::onFinalize()
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t ACT_PA10_RH::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t ACT_PA10_RH::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

RTC::ReturnCode_t ACT_PA10_RH::onActivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t ACT_PA10_RH::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t ACT_PA10_RH::onExecute(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t ACT_PA10_RH::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t ACT_PA10_RH::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t ACT_PA10_RH::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t ACT_PA10_RH::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t ACT_PA10_RH::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


extern "C"
{
 
  void ACT_PA10_RHInit(RTC::Manager* manager)
  {
    coil::Properties profile(act_pa10_rh_spec);
    manager->registerFactory(profile,
                             RTC::Create<ACT_PA10_RH>,
                             RTC::Delete<ACT_PA10_RH>);
  }
  
};



