// -*- C++ -*-
/*!
 * @file  HandCtrl.cpp
 * @brief open/close controlled RH707 hand
 * @date $Date$
 *
 * $Id$
 */

#include "HandCtrl.h"

// Module specification
// <rtc-template block="module_spec">
static const char* handctrl_spec[] =
  {
    "implementation_id", "HandCtrl",
    "type_name",         "HandCtrl",
    "description",       "open/close controlled RH707 hand",
    "version",           "1.0.0",
    "vendor",            "FSI",
    "category",          "hand",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
HandCtrl::HandCtrl(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_RH707ProvPort("RH707Prov")

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
HandCtrl::~HandCtrl()
{
}



RTC::ReturnCode_t HandCtrl::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  
  // Set OutPort buffer
  
  // Set service provider to Ports
  m_RH707ProvPort.registerProvider("com_hand", "ComHand", m_com_hand);
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  addPort(m_RH707ProvPort);
  
  // </rtc-template>

 if(m_com_hand.InitHand() != 0){
	return RTC::RTC_ERROR;
  }
  return RTC::RTC_OK;
}


RTC::ReturnCode_t HandCtrl::onFinalize()
{
  int err;
  
  err = DioClose(m_com_hand.hDrv);							// Close process
  if(err != 0) {
	  RTC_ERROR(("DioClose error!"));
	  return RTC::RTC_ERROR;
  }
  RTC_ERROR(("Dio Closed."));
  m_com_hand.hDrv = INVALID_HANDLE_VALUE;

  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t HandCtrl::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t HandCtrl::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t HandCtrl::onActivated(RTC::UniqueId ec_id)
{
  int err;

  err = DioOpen(&m_com_hand.hDrv, 2, 1);				// Open process
  if(err != 0) {
	  RTC_ERROR(("DioOpen error!"));
	  return RTC::RTC_ERROR;
  }
  RTC_ERROR(("Dio Opened."));

  return RTC::RTC_OK;
}


RTC::ReturnCode_t HandCtrl::onDeactivated(RTC::UniqueId ec_id)
{
  int err;
  
  err = DioClose(m_com_hand.hDrv);							// Close process
  if(err != 0) {
	  RTC_ERROR(("DioClose error!"));
	  return RTC::RTC_ERROR;
  }
  RTC_ERROR(("Dio Closed."));
  m_com_hand.hDrv = INVALID_HANDLE_VALUE;

  return RTC::RTC_OK;
}


RTC::ReturnCode_t HandCtrl::onExecute(RTC::UniqueId ec_id)
{ 
  int err;

  // when hand is moving
  if(m_com_hand.cond == 1){
	  m_com_hand.InpPortNo[0] = 1;								// Set port 0 to InpPortNo[0]
	  m_com_hand.lpDInp.InpPortNo = m_com_hand.InpPortNo;		// Set the pointer of InpPortNo
	  m_com_hand.lpDInp.PortNum = 1;							// Total numbe of logical ports
	  m_com_hand.lpDInp.Buf = m_com_hand.BufIn;				// Set the pointer of Buf
	  err = DioInp(m_com_hand.hDrv,&m_com_hand.lpDInp);
	  if(err != 0){
		  RTC_ERROR(("DioInp error!: %x\n",err));
		  return RTC::RTC_ERROR;
	  }
	  if(m_com_hand.lpDInp.Buf[0] != m_com_hand.old_bit && m_com_hand.lpDInp.Buf[0] != 0){
		  switch(m_com_hand.lpDInp.Buf[0]){
			case(0x2):// OPENED
				m_com_hand.cond = 3;
				break;
			case(0x1):// CLOSED
			    m_com_hand.cond = 2;
				break;
			case(0x4):// INTERRUPTED
			    m_com_hand.cond = 4;
				break;
			default:
				RTC_ERROR(("UNKNOWN CONDITION! (Neither \"OPENED\",\"CLOSED\",nor \"INTERRUPTED\""));
				return RTC::RTC_ERROR;
		  }
		  m_com_hand.old_bit = m_com_hand.lpDInp.Buf[0];
	  }
  }
  
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t HandCtrl::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t HandCtrl::onError(RTC::UniqueId ec_id)
{

  return RTC::RTC_OK;
}


RTC::ReturnCode_t HandCtrl::onReset(RTC::UniqueId ec_id)
{
  
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t HandCtrl::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t HandCtrl::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void HandCtrlInit(RTC::Manager* manager)
  {
    coil::Properties profile(handctrl_spec);
    manager->registerFactory(profile,
                             RTC::Create<HandCtrl>,
                             RTC::Delete<HandCtrl>);
  }
  
};


